@@ -370,7 +370,9 @@ pixman_rasterize_edges (pixman_image_t *image,
                         pixman_fixed_t  t,
                         pixman_fixed_t  b)
 {
-    if (image->common.read_func || image->common.write_func)
+    return_if_fail (image->type == BITS);
+    
+    if (image->bits.read_func || image->bits.write_func)
 	pixman_rasterize_edges_accessors (image, l, r, t, b);
     else
 	pixman_rasterize_edges_no_accessors (image, l, r, t, b);
